﻿'
' Copyright (c) 2021, MindFusion LLC - Bulgaria.
'

Imports System.IO

Imports MindFusion.Diagramming.Wpf.Layout


Class MainWindow

	Public Sub New()
        InitializeComponent()

        dirDiagram.ModificationStart = ModificationStart.AutoHandles
        dirDiagram.Behavior = Behavior.Modify

		dirDiagram.ShowAnchors = ShowAnchors.Never
		folderAnchors = New AnchorPattern(New AnchorPoint() _
		{ _
		 New AnchorPoint(10, 100, False, True), _
		 New AnchorPoint(0, 50, True, False) _
		  }, "FolderAnchors")

		' set folder-like polygonal shape as default for nodes
		Dim folderShape As Byte() = {0, 0, 40, 0, 55, 20, _
		 100, 20, 100, 100, 0, 100}
		dirDiagram.DefaultShape = New Shape(folderShape)

		' allow hierarchical expand/collapse
		dirDiagram.NodesExpandable = True

		' all arrows should start with vertical segment
		dirDiagram.LinkCascadeOrientation = Orientation.Vertical
        dirDiagram.LinkHeadShape = ArrowHeads.None

        dirDiagram.ExpandButtonPosition = ExpandButtonPosition.OuterUpperLeft

		' read some directories
		BuildTree()
	End Sub

	Private Shadows height As Integer = 0
	Private folderAnchors As AnchorPattern

	Private Sub BuildTree()
		' create a box that will be a root in our hierarchy
		Dim root As New ShapeNode(dirDiagram)
		root.AnchorPattern = folderAnchors
		root.Bounds = New Rect(23.04, 23.04, 99.84, 46.08)
		root.Text = "root"
		dirDiagram.Nodes.Add(root)

		height = 20

		' read directories of all drives
		Dim drives As String() = Environment.GetLogicalDrives()

		Dim skip As Integer = 0
		If drives.Length >= 2 Then
			If drives(0)(0) = "A"c Then
				skip += 1
			End If
			If drives(1)(0) = "B"c Then
				skip += 1
			End If
		End If

		Dim di As DirectoryInfo() = New DirectoryInfo(drives.Length - skip - 1) {}
		For i As Integer = 0 To di.GetLength(0) - 1
			di(i) = New DirectoryInfo(drives(i + skip))
		Next
		BuildSubDirs(root, di, 1)

		LayoutTree()

		' after the whole tree is built adjust the document extents
		' to be as big as needed
		dirDiagram.ResizeToFitItems(5, True)
	End Sub

	Private Sub LayoutTree()
		Dim tl As New TreeLayout()

		tl.Root = Nothing
		' automatically select root
		tl.Type = TreeLayoutType.Cascading
		tl.Direction = TreeLayoutDirections.LeftToRight
		tl.LinkStyle = TreeLayoutLinkType.Cascading2
		tl.LevelDistance = -40
		tl.NodeDistance = 15
		tl.KeepRootPosition = True
		tl.ReversedLinks = False
		tl.Anchoring = Anchoring.Keep

		tl.Arrange(dirDiagram)
	End Sub

	Private Sub BuildSubDirs(ByVal root As ShapeNode, ByVal dirs As DirectoryInfo(), ByVal level As Integer)
		For Each dir As DirectoryInfo In dirs
			' create a node for the subfolder
			Dim folder As ShapeNode = dirDiagram.Factory.CreateShapeNode(6 + level * 76.8, height, 99.84, 46.08)
			folder.AnchorPattern = folderAnchors
			folder.Text = dir.Name
			folder.Tag = dir
			' associate dir-info with box
			dirDiagram.Nodes.Add(folder)

			' link the subfolder with its parent
			Dim link As New DiagramLink(dirDiagram, root, folder)
			link.SegmentCount = 2
			link.Shape = LinkShape.Cascading
			dirDiagram.Links.Add(link)

			' vertical offset for next folders
			height += 14
			If level > 2 Then
				Continue For
			End If
			' avoid deep recursion
			Dim subDirs As DirectoryInfo()
			Try
				' build subtrees recursively
				subDirs = dir.GetDirectories()
				BuildSubDirs(folder, subDirs, level + 1)
			Catch
			End Try
		Next
	End Sub

	Private Sub OnTreeCollapsed(ByVal sender As Object, ByVal e As NodeEventArgs)
		LayoutTree()
	End Sub

	Private Sub OnTreeExpanded(ByVal sender As Object, ByVal e As NodeEventArgs)
		LayoutTree()
	End Sub

End Class
